/* Keep these headers */
#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>
#include <tice.h>

/* Standard headers - it's recommended to leave them included */
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* Shared libraries */
#include <lib\ce\graphx.h>
#include <lib/ce/fileioc.h>
#include <lib/ce/keypadc.h>
#include "gfx\player_gfx.h"
#include "gfx\tile_gfx.h"

/* Tilemap defines */
#define TILE_WIDTH	11
#define TILE_HEIGHT	11
#define TILEMAP_WIDTH	30
#define TILEMAP_HEIGHT	18
#define TILEMAP_X_OFFSET	-5
#define TILEMAP_Y_OFFSET	-5

/* Palette defines */
#define SPIKE_COLOR	107
#define BLOCK_COLOR	227
#define WHITE	255

extern uint8_t TITOL_map[];


int player_x, player_y = 0;
uint8_t key7;
uint8_t rgb;
uint8_t block_color[3], spike_color[3];

const gfx_image_t *tile_ptrs[] = {
   block, block, up_spike, down_spike, left_spike, right_spike, start_pipe, end_pipe, door, door, button
};

/* Put function prototypes here */
void draw_tile( uint8_t tile_x, uint8_t tile_y );
void draw_tilemap(void);

/* Put all your code here */
void main(void) {
	srand(rtc_Time());
	
	/* Initialize graphics */
	gfx_Begin(gfx_8bpp);
	gfx_SetPalette(tile_gfx_pal, sizeof(tile_gfx_pal), 0);
	gfx_SetDrawBuffer();
	
	for( rgb = 0; rgb < 3; rgb++ ) {
		block_color[rgb] = randInt( 1, 255 );
	}
	gfx_palette[BLOCK_COLOR] = gfx_RGBTo1555( block_color[0], block_color[1], block_color[2] );
	for( rgb = 0; rgb < 3; rgb++ ) {
		spike_color[rgb] = randInt( 1, 255 );
	}
	gfx_palette[SPIKE_COLOR] = gfx_RGBTo1555( spike_color[0], spike_color[1], spike_color[2] );
	
	/* Program code*/
	
	while( kb_ScanGroup(kb_group_6) != kb_Clear ) {
		gfx_FillScreen( WHITE );
		draw_tilemap();
		gfx_TransparentSprite( player, player_x, player_y );
		gfx_SwapDraw();
		
		key7 = kb_ScanGroup(kb_group_7);
		
		if ( key7 & kb_Up ) {
			player_y--;
		}
		if ( key7 & kb_Down ) {
			player_y++;
		}
		if ( key7 & kb_Left ) {
			player_x--;
		}
		if ( key7 & kb_Right ) {
			player_x++;
		}
		
	}
	
	/* Clean up and exit */
	gfx_End();
	prgm_CleanUp();
}

void draw_tilemap(void) {
	uint8_t tilemap_x, tilemap_y;
	for( tilemap_x = 0; tilemap_x <= TILEMAP_WIDTH - 1; tilemap_x++ ) {
		for( tilemap_y = 0; tilemap_y <= TILEMAP_HEIGHT - 1; tilemap_y++ ) {
			draw_tile( tilemap_x, tilemap_y );
		}
	}
}

void draw_tile( uint8_t tile_x, uint8_t tile_y ) {
	int xpos = TILE_WIDTH * tile_x + TILEMAP_X_OFFSET, ypos = TILE_HEIGHT * tile_y + TILEMAP_Y_OFFSET;
	uint8_t tile = TITOL_map[TILEMAP_WIDTH * tile_y + tile_x];
	switch( tile ) {
		case 0:
			break;
		case 1:
			gfx_Sprite( block, xpos, ypos );
			break;
		case 8:
		case 9:
			gfx_TransparentSprite_NoClip( door, xpos, ypos );
			break;
		default:
			gfx_TransparentSprite_NoClip( tile_ptrs[tile], xpos, ypos );
	}
}